package de.web.databasesupport.tools.database;

import java.sql.*;

/********************************************************************
  *
  *  program     : Database.java
  *  author      : Niels-Peter de Witt
  *  version     : 2001-03-27(001)
  *  (c)         : Helge Janicke, Niels-Peter de Witt, Karsten Wolke
  *  email       : heljanic@agent.fho-emden.de
  *      : ndewitt@gmx.de
  *      : mail@karsten-wolke.de
  *  description : Interface for database access.
  *
  *******************************************************************
  * modifications: See at the end of the document.
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
  * This software is is provided AS IS, without a warranty of any
  * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATION AND
  * WARRANTIES, INCLUDING ANY IMLIED WARRANTY OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
  * HEREBY EXCLUDED. WE SHALL NOT BE LIABLE FOR ANY DAMAGES
  * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
  * DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WE
  * WILL BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR
  * DIRECT, INDIRECT, SPECIAL, CENSEQUENTIAL, INCIDENTIAL OR
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY
  * OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
  * SOFTWARE, EVEN IF WE HAVE BEEN ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGES.
  * This software is not designed or intended for use in online
  * control of aircraft, air traffic, aircraft navigation or
  * aircraft communications; or in the design, construction,
  * operation or maintenance of any nuclear facility. Licensee
  * represents and warrants that it will not use or redistribute
  * the software for such purposes.
********************************************************************/

/** This interface declares basic functionality to access a database.
 *  It's possible to retrieve information on tables and columns within
 *  a table. E.g. testing if a specific table exists in the database
 *  or what columns exist in a specific table.<br>
 *  There is also functionality to send SQL statements directly to the
 *  database or to precompile often used statements for better
 *  performance.
 *  @see java.sql.ResultSet and the java.sql package for more detailed
 *       information on how to handle ResultSets.
 */
public interface Database {

    /** Opens the connection to the database.
     *  Using the specified database and driver.
     *  @throw SQLException Cannot set AutoCommit to false.
     *  @throw ClassNotFoundException Cannot load driver class (check classpath).
     *  @throw InstantiationException Cannot generate a new instance of the driver.
     *  @throw IllegalAccessException If the driver class or initializer is not accessable.
     */
    public void open()
    throws ClassNotFoundException, SQLException, InstantiationException, IllegalAccessException;

    /** Closes the connection to the database.
     *  @throw SQLException - Cannot close database.
     *  @throw NotConnectdException - Not connected to Database.
     */

    public void close()
    throws SQLException, NotConnectedException;

    /** Commits pending data to the database.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public void commit()
    throws SQLException, NotConnectedException;

    /** Precompiles a SQL statement.
     *  Increases performance for frequently used SQL statements.
     *  A openened database connection is needed.
     *  @return The precompiled statement.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public PreparedStatement precompile(String statement)
    throws SQLException, NotConnectedException;

    /** Executes an Query from a SQL statement.
     *  A opened database connection is needed.
     *  @param statement The SQL satement to execute.
     *  @return The ResultSet returned from the database.
     *  @throw SQLExeption - if database access error occurs,
     *      or the send statement does not return a value.
     *      In the second case the errorcode of the SQLException is 0.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public ResultSet executeQuery(String statement)
    throws SQLException, NotConnectedException;

    /** Executes an Update statement.
     *  A opened database connection is needed.
     *  @param statement The SQL satement to execute.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public void executeUpdate(String statement)
    throws SQLException, NotConnectedException;

    /** Returns names of all tables within the database.
     *  @return The names of all tables.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public String[] getTablenames()
    throws SQLException, NotConnectedException;

    /** Returns a ResultSet containing description of a table.
     *  @param tableName The name of the table.
     *  @return The description for the table.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public ResultSet getTableDescription(String tableName)
    throws SQLException, NotConnectedException;

    /** Returns a ResultSet containing the descriptions of all tables.
     *  @return The description of all tables.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public ResultSet getTableDescriptions()
    throws SQLException, NotConnectedException;

    /** Returns all column names for the table.
     *  @param tableName The table name.
     *  @return All column names in the table.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public String[] getColumnNames(String tableName)
    throws SQLException, NotConnectedException;

    /** Returns the names of the Columns in a ResultSet.
     *  @param result The ResultSet to examine.
     *  @return All column names.
     *  @throw SQLException - Can't get Names from ResultSet.
     */
    public String[] getColumnNames(ResultSet result)
    throws SQLException;

    /** Returns a ResultSet containing a description of the column.
     *  @param tableName The name of the table.
     *  @param columnName The name of the column.
     *  @return The description.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public ResultSet getColumnDescription(String tableName, String columnName)
    throws SQLException, NotConnectedException;

    /** Returns a ResultSet containing a description of the column.
     *  @param tableName The name of the table.
     *  @param columnName The name of the column.
     *  @return The description.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public ResultSet getColumnDescription(String tableName, int columnIndex)
    throws SQLException, NotConnectedException, IndexOutOfBoundsException;

    /** Returns a ResultSet containing a description of all columns in  table.
     *  @param tableName The name of the table.
     *  @return The description.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public ResultSet getColumnDescriptions(String tableName)
    throws SQLException, NotConnectedException;

    /** Tests if a table exists.
     *  @param tableName The name of the table.
     *  @return <tt>true</tt> if the table exists, <tt>false</tt> otherwise.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public boolean tableExists(String tableName)
    throws SQLException, NotConnectedException;

    /** Tests if a column exists.
     *  @param tableName The name of the table.
     *  @param columnName The name of the column.
     *  @return <tt>true</tt> if the column exists in the table,
     *          <tt>false</tt> otherwise.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public boolean columnExists(String tableName, String columnName)
    throws SQLException, NotConnectedException;

    /** Tests if a column exists.
     *  @param tableName The name of the table.
     *  @param columnIndex The index of the column.
     *  @return <tt>true</tt> if the column exists in the table,
     *          <tt>false</tt> otherwise.
     *  @throw SQLException - Cannot examine if column exists in database.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public boolean columnExists(String tableName, int columnIndex)
    throws SQLException, NotConnectedException;

    /** Tests if is closed.
     *  @return <tt>true</tt> if the database connection is closed,
     *          <tt>false</tt> otherwise.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    boolean isClosed()
    throws SQLException, NotConnectedException;

    /** Returns the column count of the table.
     *  @param tableName The name of the table.
     *  @return The column count.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public int getColumnCount(String tableName)
    throws SQLException, NotConnectedException;

    /** Returns the column count of a ResultSet.
     *  @param tableName The name of the table.
     *  @return The column count.
     *  @throw SQLException - Cannot count columns.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public int getColumnCount(ResultSet result)
    throws SQLException;

    /** Returns the table count of the database.
     *  @return The table count.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectdException - Not connected to Database.
     */
    public int getTableCount()
    throws SQLException, NotConnectedException ;

    /** Returns the names of the Columns in a ResultSet.
     *  @param result The ResultSet to examine.
     *  @return All column names.
     *  @throw SQLException - if database access error occurs.
     */
    public java.util.Vector getColumnNamesVector(ResultSet result)
    throws SQLException;

    /** Returns all column names for the table.
     *  @param tableName The table name.
     *  @return All column names in the table.
     *  @throw SQLExeption - if database access error occurs.
     *  @throw NotConnectedException - if database is closed.
     */
    public java.util.Vector getColumnNamesVector(String tableName)
    throws SQLException, NotConnectedException;

     /** Returns names of all tables within the database.
         *  @return The names of all tables.
         *  @throw SQLExeption - if database access error occurs.
         *  @throw NotConnectedException - if database is closed.
         */
    public java.util.Vector getTablenamesVector()
    throws SQLException, NotConnectedException ;


} // end of Database.java

/********************************************************************
  * date: ******* who: ****** description: **************************
  * -----------------------------------------------------------------
  * 2001-03-27    hel & nd    designing the interface
  * -----------------------------------------------------------------
  * 2001-03-28    heljanic       documenting the interface.
  *                                          changed following method declarations:
  *                                             - getColumnCount(): inserted param String tableName.
  *                                             - columnExists()  : inserted param String tableName.
  * -----------------------------------------------------------------
  * 2001-03-28    niewitt       throw statements.
  * -----------------------------------------------------------------
  * 2001-03-29    niewitt       inserted getColumnNames(ResultSet).
  *                                         changed following methods:
  *                                         - getColumnCount(ResultSet): deleted throws
  *                                                NotConnctedException
  * 2001-03-31      niewitt     inserted Vector getColumnNamesVector(String)
  *                                          inserted Vector getTableNamesVector()
  *                                       inserted Vector getColumnNamesVector(Result)
  * -----------------------------------------------------------------
  * 2001-04-01      niewitt     renamed executeStatement() to
  *                               executeQuery()
  *                             inserted executeUodate()
  * -----------------------------------------------------------------

********************************************************************/






