package de.web.databasesupport.tools.iotools;
import java.io.*;
/********************************************************************
  *
  *  program     : ASCIIFile.java
  *  author      : Niels-Peter de Witt
  *  version     : 2001-04-12(001)
  *  (c)         : Helge Janicke, Niels-Peter de Witt, Karsten Wolke
  *  email       : heljanic@agent.fho-emden.de
  *              : ndewitt@gmx.de
  *              : mail@karsten-wolke.de
  *  description : Handles ASCII-files
  *
  *******************************************************************
  * modifications: See at the end of the document.
  *******************************************************************
  * comments:
  *******************************************************************
  * license:
  * This software is provided AS IS, without a warranty of any
  * kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATION AND
  * WARRANTIES, INCLUDING ANY IMLIED WARRANTY OF MERCHANTABILITY,
  * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
  * HEREBY EXCLUDED. WE SHALL NOT BE LIABLE FOR ANY DAMAGES
  * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
  * DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WE
  * WILL BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR
  * DIRECT, INDIRECT, SPECIAL, CENSEQUENTIAL, INCIDENTIAL OR
  * PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY
  * OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE
  * SOFTWARE, EVEN IF WE HAVE BEEN ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGES.
  * This software is not designed or intended for use in online
  * control of aircraft, air traffic, aircraft navigation or
  * aircraft communications; or in the design, construction,
  * operation or maintenance of any nuclear facility. Licensee
  * represents and warrants that it will not use or redistribute
  * the software for such purposes.
********************************************************************/

 /** handles ASCII-files
    */
public class ASCIIFile {

    public ASCIIFile() {

    }

    /** Writes a text to the spezified File, if allowed.
    * @param dest Destination-file to write the text in.
    * @param text Text to put in the file.
    * @param overwrite Flag if allowed to overwrite an existing file (<tt>true</tt> if alowed, <tt>false</tt> otherwise).
    * @return true if writing was succesfull, false if file aready exists and it is not allowed to overwrite it.
    * @throw IOException - if something went wrong with the writing, or creating of the destination-file.
    */
    public static synchronized boolean write(File dest, String text, boolean overwrite)
    throws IOException {
        BufferedWriter out= new BufferedWriter( new OutputStreamWriter( new FileOutputStream(dest)));
        if (overwrite) {
            out.write(text);
            out.close();
            return true;
        } else {
            return false;
        }
    }

    /** Reads a text from the spezified File and returns it as one big String.
    * @param source Source-file to to read from.
    * @return The text as one big String.
    * @throw IOException - if something went wrong with the reading from the file.
    */
    public static synchronized String read(File source)
    throws IOException {
        StringBuffer rv = new StringBuffer();
        BufferedReader in = new BufferedReader ( new InputStreamReader ( new FileInputStream(source)));
        if (in.ready()) {
            String text= "";
            while( (text = in.readLine()) != null) {
                rv.append(text+"\n");
            }
        }
        in.close();
        return rv.toString();
    }

    /** Reads a text from the spezified File and returns it as a Vector of Strings.
    * each line is represented through one String-object in the Vector.
    * @param source Source-file to to read from.
    * @return The Vector of Strings, where each String-object represents a Line.
    * @throw IOException - if something went wrong with the reading from the file.
    */
    public static synchronized java.util.Vector readAsVector(File source)
    throws IOException {
        java.util.Vector vec = new java.util.Vector();
        vec.clear();
        BufferedReader in = new BufferedReader ( new InputStreamReader ( new FileInputStream(source)));
        while (in.ready()) {
            String text= "";
            while( (text = in.readLine())!=null) {
                vec.add(text);
            }
        }
        in.close();
        return vec;
    }

    /** Reads a text from the spezified File and returns it as a String-array.
    * each line is represented as one field in the array.
    * @param source Source-file to to read from.
    * @return The Array of Strings, where each String-object represents a Line.
    * @throw IOException - if something went wrong with the reading from the file.
    */
    public static synchronized String[] readAsArray(File source)
    throws IOException {
        java.util.Vector vec = new java.util.Vector();
        vec.clear();
        vec = readAsVector(source);
        String[] rv = new String[vec.size()];
        for (int i =0; i<vec.size(); i++) {
            rv[i] = vec.get(i).toString();
        }
        return rv;
    }

    /** Appends a text at a spezified file.
    * @param dest Destination-file to to append the text at.
    * @param text The text to append.
    * @return <tt>true</tt> if appending was successfull, or <tt>false<tt> if something went wrong with reading or writing.
    */
    public static synchronized boolean append(File dest, String text) {
        boolean rv = false;
        try {
            StringBuffer textnew = new StringBuffer(ASCIIFile.read(dest));
            textnew.append(text);
            ASCIIFile.write(dest, textnew.toString(), true);
            rv = true;
        } catch (IOException exc) {
            rv = false;
        }
        return rv;
    }

    /** Inserts a text at the given linenumber of the spezified file.
    * @param dest Destiantion-file where to insert the text.
    * @param text The text to insert.
    * @param linenumber The linenumber where to insert the text.
    * @return <tt>true</tt> if inserting was successfull, <tt>false</tt> if something went wrong with reading or writing.
    * @throw IndexOutOfBoundsException - if the spezified linenumber wasn't in the Range of the available lines of the file.
    */
    public static synchronized boolean insert(File dest, String text, int linenumber)
    throws IndexOutOfBoundsException {
        boolean rv = false;
        StringBuffer newtext = new StringBuffer();
        try {
            java.util.Vector vec = new java.util.Vector();
            vec = readAsVector(dest);
            vec.insertElementAt(text,linenumber);
            for ( int i=0; i<vec.size(); i++) {
                newtext.append(vec.get(i)+"\n");
            }
            write(dest, newtext.toString(), true);
            rv = true;
        } catch (IOException exc) {
            rv = false;
        }
        return rv;
    }

 /** Counts the available lines of the given file.
    * @param source The Source-file to count the lines of.
    * @return The number of lines in the spezified file
    * @throw IOException - if something went wrong with the reading of the file.
    */
    public static int countLines(File source)
    throws IOException {
        int count = 0;
        String[] temp = readAsArray(source);
        count = temp.length;
        return count;
    }

}